@@ -12,8 +12,7 @@ const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
+const _ = ExtensionUtils.gettext;
 
 const appSys = Shell.AppSystem.get_default();
 
@@ -25,10 +24,13 @@ const NAVIGATION_REGION_OVERSHOOT = 50;
 Gio._promisify(Gio._LocalFilePrototype, 'query_info_async', 'query_info_finish');
 Gio._promisify(Gio._LocalFilePrototype, 'set_attributes_async', 'set_attributes_finish');
 
-var ApplicationMenuItem = GObject.registerClass(
 class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, app) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, app) {
+        super();
         this._app = app;
         this._button = button;
 
@@ -94,12 +96,15 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         icon.style_class = 'icon-dropshadow';
         this._iconBin.set_child(icon);
     }
-});
+}
 
-var CategoryMenuItem = GObject.registerClass(
 class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, category) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, category) {
+        super();
         this._category = category;
         this._button = button;
 
@@ -181,11 +186,14 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
     }
 
     _onMotionEvent(actor, event) {
-        let device = event.get_device();
-        if (!device.get_grabbed_actor()) {
+        if (!this._grab) {
             this._oldX = -1;
             this._oldY = -1;
-            device.grab(this);
+            const grab = global.stage.grab(this);
+            if (grab.get_seat_state() !== Clutter.GrabState.NONE)
+                this._grab = grab;
+            else
+                grab.dismiss();
         }
         this.hover = true;
 
@@ -195,7 +203,8 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._oldX = -1;
         this._oldY = -1;
         this.hover = false;
-        device.ungrab();
+        this._grab?.dismiss();
+        delete this._grab;
 
         let source = event.get_source();
         if (source instanceof St.Widget)
@@ -211,7 +220,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._button.selectCategory(this._category);
         this._button.scrollToCatButton(this);
     }
-});
+}
 
 class ApplicationsMenu extends PopupMenu.PopupMenu {
     constructor(sourceActor, arrowAlignment, arrowSide, button) {
@@ -350,10 +359,13 @@ class DesktopTarget {
 }
 Signals.addSignalMethods(DesktopTarget.prototype);
 
-let ApplicationsButton = GObject.registerClass(
 class ApplicationsButton extends PanelMenu.Button {
-    _init() {
-        super._init(1.0, null, false);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(1.0, null, false);
 
         this.setMenu(new ApplicationsMenu(this, 1.0, St.Side.TOP, this));
         Main.panel.menuManager.addMenu(this.menu);
@@ -363,17 +375,13 @@ class ApplicationsButton extends PanelMenu.Button {
         // role ATK_ROLE_MENU like other elements of the panel.
         this.accessible_role = Atk.Role.LABEL;
 
-        let hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
-
         this._label = new St.Label({
             text: _('Applications'),
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
         });
-        hbox.add_child(this._label);
-        hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
 
-        this.add_actor(hbox);
+        this.add_actor(this._label);
         this.name = 'panelApplications';
         this.label_actor = this._label;
 
@@ -383,9 +391,12 @@ class ApplicationsButton extends PanelMenu.Button {
         this._hidingId = Main.overview.connect('hiding', () => {
             this.remove_accessible_state(Atk.StateType.CHECKED);
         });
-        Main.layoutManager.connect('startup-complete',
-            this._setKeybinding.bind(this));
-        this._setKeybinding();
+        Main.wm.addKeybinding(
+            'apps-menu-toggle-menu',
+            ExtensionUtils.getSettings(),
+            Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
+            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
+            () => this.menu.toggle());
 
         this._desktopTarget = new DesktopTarget();
         this._desktopTarget.connect('app-dropped', () => {
@@ -436,11 +447,7 @@ class ApplicationsButton extends PanelMenu.Button {
         this._tree.disconnect(this._treeChangedId);
         this._tree = null;
 
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            Main.sessionMode.hasOverview
-                ? Main.overview.toggle.bind(Main.overview)
-                : null);
+        Main.wm.removeKeybinding('apps-menu-toggle-menu');
 
         this._desktopTarget.destroy();
     }
@@ -482,12 +489,6 @@ class ApplicationsButton extends PanelMenu.Button {
         super._onOpenStateChanged(menu, open);
     }
 
-    _setKeybinding() {
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            () => this.menu.toggle());
-    }
-
     _redisplay() {
         this.applicationsBox.destroy_all_children();
         this.categoriesBox.destroy_all_children();
@@ -673,21 +674,24 @@ class ApplicationsButton extends PanelMenu.Button {
 
         return applist;
     }
-});
+}
 
 let appsMenuButton;
 
+/** */
 function enable() {
     appsMenuButton = new ApplicationsButton();
     let index = Main.sessionMode.panel.left.indexOf('activities') + 1;
     Main.panel.addToStatusArea('apps-menu', appsMenuButton, index, 'left');
 }
 
+/** */
 function disable() {
     Main.panel.menuManager.removeMenu(appsMenuButton.menu);
     appsMenuButton.destroy();
 }
 
+/** */
 function init() {
     ExtensionUtils.initTranslations();
 }
