@@ -1,17 +1,27 @@
 /* exported WindowPicker, WindowPickerToggle */
-const { Clutter, GLib, GObject, Meta, Shell, St } = imports.gi;
+const { Clutter, GObject, Shell, St } = imports.gi;
 
 const Layout = imports.ui.layout;
 const Main = imports.ui.main;
-const Overview = imports.ui.overview;
 const { WorkspacesDisplay } = imports.ui.workspacesView;
+const Workspace = imports.ui.workspace;
+
+const { VIGNETTE_BRIGHTNESS } = imports.ui.lightbox;
+const {
+    SIDE_CONTROLS_ANIMATION_TIME,
+    OverviewAdjustment,
+    ControlsState,
+} = imports.ui.overviewControls;
 
-let MyWorkspacesDisplay = GObject.registerClass(
 class MyWorkspacesDisplay extends WorkspacesDisplay {
-    _init() {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(controls, overviewAdjustment) {
         let workspaceManager = global.workspace_manager;
 
-        this._workspaceAdjustment = new St.Adjustment({
+        const workspaceAdjustment = new St.Adjustment({
             value: workspaceManager.get_active_workspace_index(),
             lower: 0,
             page_increment: 1,
@@ -20,32 +30,29 @@ class MyWorkspacesDisplay extends WorkspacesDisplay {
             upper: workspaceManager.n_workspaces,
         });
 
+        super(controls, workspaceAdjustment, overviewAdjustment);
+
+        this._workspaceAdjustment = workspaceAdjustment;
+        this._workspaceAdjustment.actor = this;
+
         this._nWorkspacesChangedId =
             workspaceManager.connect('notify::n-workspaces',
                 this._updateAdjustment.bind(this));
 
-        super._init(this._workspaceAdjustment);
-
-        this._workspaceAdjustment.actor = this;
-
         this.add_constraint(
             new Layout.MonitorConstraint({
                 primary: true,
                 work_area: true,
             }));
-
-        this._workareasChangedId = global.display.connect('workareas-changed',
-            this._onWorkAreasChanged.bind(this));
-        this._onWorkAreasChanged();
     }
 
-    animateToOverview(...args) {
+    prepareToEnterOverview(...args) {
         if (!this._scrollEventId) {
             this._scrollEventId = Main.windowPicker.connect('scroll-event',
                 this._onScrollEvent.bind(this));
         }
 
-        super.animateToOverview(...args);
+        super.prepareToEnterOverview(...args);
     }
 
     vfunc_hide(...args) {
@@ -56,13 +63,6 @@ class MyWorkspacesDisplay extends WorkspacesDisplay {
         super.vfunc_hide(...args);
     }
 
-    _onWorkAreasChanged() {
-        let { primaryIndex } = Main.layoutManager;
-        this._actualGeometry =
-            Main.layoutManager.getWorkAreaForMonitor(primaryIndex);
-        this._syncWorkspacesActualGeometry();
-    }
-
     _updateAdjustment() {
         let workspaceManager = global.workspace_manager;
         this._workspaceAdjustment.set({
@@ -71,41 +71,103 @@ class MyWorkspacesDisplay extends WorkspacesDisplay {
         });
     }
 
-    _updateWorkspacesViews() {
-        super._updateWorkspacesViews();
-
-        this._workspacesViews.forEach(v => {
-            Main.layoutManager.overviewGroup.remove_actor(v);
-            Main.windowPicker.add_actor(v);
-        });
-    }
-
     _onDestroy() {
-        if (this._workareasChangedId)
-            global.display.disconnect(this._workareasChangedId);
-        this._workareasChangedId = 0;
-
         if (this._nWorkspacesChangedId)
             global.workspace_manager.disconnect(this._nWorkspacesChangedId);
         this._nWorkspacesChangedId = 0;
 
         super._onDestroy();
     }
-});
+}
 
-var WindowPicker = GObject.registerClass({
-    Signals: {
+class MyWorkspace extends Workspace.Workspace {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(...args) {
+        super(...args);
+
+        this._adjChangedId =
+            this._overviewAdjustment.connect('notify::value', () => {
+                const { value: progress } = this._overviewAdjustment;
+                const brightness = 1 - (1 - VIGNETTE_BRIGHTNESS) * progress;
+                for (const bg of this._background?._backgroundGroup ?? []) {
+                    bg.content.set({
+                        vignette: true,
+                        brightness,
+                    });
+                }
+            });
+    }
+
+    _onDestroy() {
+        super._onDestroy();
+
+        if (this._adjChangedId)
+            this._overviewAdjustment.disconnect(this._adjChangedId);
+        this._adjChangedId = 0;
+    }
+}
+
+class MyWorkspaceBackground extends Workspace.WorkspaceBackground {
+    static {
+        GObject.registerClass(this);
+    }
+
+    _updateBorderRadius() {
+    }
+
+    vfunc_allocate(box) {
+        this.set_allocation(box);
+
+        const themeNode = this.get_theme_node();
+        const contentBox = themeNode.get_content_box(box);
+
+        this._bin.allocate(contentBox);
+
+        const [contentWidth, contentHeight] = contentBox.get_size();
+        const monitor = Main.layoutManager.monitors[this._monitorIndex];
+        const xRatio = contentWidth / this._workarea.width;
+        const yRatio = contentHeight / this._workarea.height;
+
+        const right = area => area.x + area.width;
+        const bottom = area => area.y + area.height;
+
+        const offsets = {
+            left: xRatio * (this._workarea.x - monitor.x),
+            right: xRatio * (right(monitor) - right(this._workarea)),
+            top: yRatio * (this._workarea.y - monitor.y),
+            bottom: yRatio * (bottom(monitor) - bottom(this._workarea)),
+        };
+
+        contentBox.set_origin(-offsets.left, -offsets.top);
+        contentBox.set_size(
+            offsets.left + contentWidth + offsets.right,
+            offsets.top + contentHeight + offsets.bottom);
+        this._backgroundGroup.allocate(contentBox);
+    }
+}
+
+var WindowPicker = class WindowPicker extends Clutter.Actor {
+    static [GObject.signals] = {
         'open-state-changed': { param_types: [GObject.TYPE_BOOLEAN] },
-    },
-}, class extends Clutter.Actor {
-    _init() {
+    };
+
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super({ reactive: true });
+
         this._visible = false;
         this._modal = false;
 
         this._overlayKeyId = 0;
         this._stageKeyPressId = 0;
 
-        super._init();
+        this._adjustment = new OverviewAdjustment(this);
 
         this.connect('destroy', this._onDestroy.bind(this));
 
@@ -116,31 +178,14 @@ var WindowPicker = GObject.registerClass({
             this, 'height',
             GObject.BindingFlags.SYNC_CREATE);
 
-        this._backgroundGroup = new Meta.BackgroundGroup({ reactive: true });
-        this.add_child(this._backgroundGroup);
-
-        this._backgroundGroup.connect('scroll-event', (a, ev) => {
-            this.emit('scroll-event', ev);
-        });
-
-        // Trick WorkspacesDisplay constructor into adding actions here
-        let addActionOrig = Main.overview.addAction;
-        Main.overview.addAction = a => this._backgroundGroup.add_action(a);
-
-        this._workspacesDisplay = new MyWorkspacesDisplay();
+        this._workspacesDisplay = new MyWorkspacesDisplay(this, this._adjustment);
         this.add_child(this._workspacesDisplay);
 
-        Main.overview.addAction = addActionOrig;
-
-        this._bgManagers = [];
-
-        this._monitorsChangedId = Main.layoutManager.connect('monitors-changed',
-            this._updateBackgrounds.bind(this));
-        this._updateBackgrounds();
-
         Main.uiGroup.insert_child_below(this, global.window_group);
 
         if (!Main.sessionMode.hasOverview) {
+            this._injectBackgroundShade();
+
             this._overlayKeyId = global.display.connect('overlay-key', () => {
                 if (!this._visible)
                     this.open();
@@ -150,6 +195,14 @@ var WindowPicker = GObject.registerClass({
         }
     }
 
+    _injectBackgroundShade() {
+        this._origWorkspace = Workspace.Workspace;
+        this._origWorkspaceBackground = Workspace.WorkspaceBackground;
+
+        Workspace.Workspace = MyWorkspace;
+        Workspace.WorkspaceBackground = MyWorkspaceBackground;
+    }
+
     get visible() {
         return this._visible;
     }
@@ -164,9 +217,15 @@ var WindowPicker = GObject.registerClass({
             return;
 
         this._fakeOverviewVisible(true);
-        this._shadeBackgrounds();
-        this._fakeOverviewAnimation();
-        this._workspacesDisplay.animateToOverview(false);
+        this._workspacesDisplay.prepareToEnterOverview();
+        Main.overview._animationInProgress = true;
+
+        this._adjustment.value = ControlsState.HIDDEN;
+        this._adjustment.ease(ControlsState.WINDOW_PICKER, {
+            duration: SIDE_CONTROLS_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => (Main.overview._animationInProgress = false),
+        });
 
         this._stageKeyPressId = global.stage.connect('key-press-event',
             (a, event) => {
@@ -190,11 +249,17 @@ var WindowPicker = GObject.registerClass({
         if (!this._syncGrab())
             return;
 
-        this._workspacesDisplay.animateFromOverview(false);
-        this._unshadeBackgrounds();
-        this._fakeOverviewAnimation(() => {
-            this._workspacesDisplay.hide();
-            this._fakeOverviewVisible(false);
+        this._workspacesDisplay.prepareToLeaveOverview();
+
+        Main.overview._animationInProgress = true;
+        this._adjustment.ease(ControlsState.HIDDEN, {
+            duration: SIDE_CONTROLS_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => {
+                Main.overview._animationInProgress = false;
+                this._workspacesDisplay.hide();
+                this._fakeOverviewVisible(false);
+            },
         });
 
         global.stage.disconnect(this._stageKeyPressId);
@@ -203,16 +268,8 @@ var WindowPicker = GObject.registerClass({
         this.emit('open-state-changed', this._visible);
     }
 
-    _fakeOverviewAnimation(onComplete) {
-        Main.overview._animationInProgress = true;
-        GLib.timeout_add(
-            GLib.PRIORITY_DEFAULT,
-            Overview.ANIMATION_TIME,
-            () => {
-                Main.overview._animationInProgress = false;
-                if (onComplete)
-                    onComplete();
-            });
+    getWorkspacesBoxForState() {
+        return this.allocation;
     }
 
     _fakeOverviewVisible(visible) {
@@ -229,22 +286,32 @@ var WindowPicker = GObject.registerClass({
             if (this._modal)
                 return true;
 
-            this._modal = Main.pushModal(this, {
+            const grab = Main.pushModal(global.stage, {
                 actionMode: Shell.ActionMode.OVERVIEW,
             });
-
-            if (!this._modal) {
+            if (grab.get_seat_state() !== Clutter.GrabState.NONE) {
+                this._grab = grab;
+                this._modal = true;
+            } else {
+                Main.popModal(grab);
                 this.hide();
                 return false;
             }
         } else if (this._modal) {
-            Main.popModal(this);
+            Main.popModal(this._grab);
             this._modal = false;
+            this._grab = null;
         }
         return true;
     }
 
     _onDestroy() {
+        if (this._origWorkspace)
+            Workspace.Workspace = this._origWorkspace;
+
+        if (this._origWorkspaceBackground)
+            Workspace.WorkspaceBackground = this._origWorkspaceBackground;
+
         if (this._monitorsChangedId)
             Main.layoutManager.disconnect(this._monitorsChangedId);
         this._monitorsChangedId = 0;
@@ -257,23 +324,14 @@ var WindowPicker = GObject.registerClass({
             global.stage.disconnect(this._stageKeyPressId);
         this._stageKeyPressId = 0;
     }
+};
 
-    _updateBackgrounds() {
-        Main.overview._updateBackgrounds.call(this);
+var WindowPickerToggle = class WindowPickerToggle extends St.Button {
+    static {
+        GObject.registerClass(this);
     }
 
-    _shadeBackgrounds() {
-        Main.overview._shadeBackgrounds.call(this);
-    }
-
-    _unshadeBackgrounds() {
-        Main.overview._unshadeBackgrounds.call(this);
-    }
-});
-
-var WindowPickerToggle = GObject.registerClass(
-class WindowPickerToggle extends St.Button {
-    _init() {
+    constructor() {
         let iconBin = new St.Widget({
             layout_manager: new Clutter.BinLayout(),
         });
@@ -285,7 +343,7 @@ class WindowPickerToggle extends St.Button {
             x_align: Clutter.ActorAlign.CENTER,
             y_align: Clutter.ActorAlign.CENTER,
         }));
-        super._init({
+        super({
             style_class: 'window-picker-toggle',
             child: iconBin,
             visible: !Main.sessionMode.hasOverview,
@@ -303,4 +361,4 @@ class WindowPickerToggle extends St.Button {
             this.checked = Main.windowPicker.visible;
         });
     }
-});
+};
