@@ -2,17 +2,25 @@
 const { Clutter, Gio, GObject, Meta, St } = imports.gi;
 
 const DND = imports.ui.dnd;
+const ExtensionUtils = imports.misc.extensionUtils;
 const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
+const _ = ExtensionUtils.gettext;
+
+const TOOLTIP_OFFSET = 6;
+const TOOLTIP_ANIMATION_TIME = 150;
+
+const MAX_THUMBNAILS = 6;
 
-let WindowPreview = GObject.registerClass(
 class WindowPreview extends St.Button {
-    _init(window) {
-        super._init({
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(window) {
+        super({
             style_class: 'window-list-window-preview',
         });
 
@@ -64,10 +72,13 @@ class WindowPreview extends St.Button {
             this._window.window_type !== Meta.WindowType.DESKTOP &&
             this._window.showing_on_its_workspace();
     }
-});
+}
 
-let WorkspaceLayout = GObject.registerClass(
 class WorkspaceLayout extends Clutter.LayoutManager {
+    static {
+        GObject.registerClass(this);
+    }
+
     vfunc_get_preferred_width() {
         return [0, 0];
     }
@@ -94,12 +105,15 @@ class WorkspaceLayout extends Clutter.LayoutManager {
             child.allocate(childBox);
         }
     }
-});
+}
 
-let WorkspaceThumbnail = GObject.registerClass(
 class WorkspaceThumbnail extends St.Button {
-    _init(index) {
-        super._init({
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(index) {
+        super({
             style_class: 'workspace',
             child: new Clutter.Actor({
                 layout_manager: new WorkspaceLayout(),
@@ -107,7 +121,14 @@ class WorkspaceThumbnail extends St.Button {
             }),
         });
 
+        this._tooltip = new St.Label({
+            style_class: 'dash-label',
+            visible: false,
+        });
+        Main.uiGroup.add_child(this._tooltip);
+
         this.connect('destroy', this._onDestroy.bind(this));
+        this.connect('notify::hover', this._syncTooltip.bind(this));
 
         this._index = index;
         this._delegate = this; // needed for DND
@@ -192,19 +213,55 @@ class WorkspaceThumbnail extends St.Button {
             ws.activate(global.get_current_time());
     }
 
+    _syncTooltip() {
+        if (this.hover) {
+            this._tooltip.set({
+                text: Meta.prefs_get_workspace_name(this._index),
+                visible: true,
+                opacity: 0,
+            });
+
+            const [stageX, stageY] = this.get_transformed_position();
+            const thumbWidth = this.allocation.get_width();
+            const tipWidth = this._tooltip.width;
+            const tipHeight = this._tooltip.height;
+            const xOffset = Math.floor((thumbWidth - tipWidth) / 2);
+            const monitor = Main.layoutManager.findMonitorForActor(this);
+            const x = Math.clamp(
+                stageX + xOffset,
+                monitor.x,
+                monitor.x + monitor.width - tipWidth);
+            const y = stageY - tipHeight - TOOLTIP_OFFSET;
+            this._tooltip.set_position(x, y);
+        }
+
+        this._tooltip.ease({
+            opacity: this.hover ? 255 : 0,
+            duration: TOOLTIP_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => (this._tooltip.visible = this.hover),
+        });
+    }
+
     _onDestroy() {
+        this._tooltip.destroy();
+
         this._workspace.disconnect(this._windowAddedId);
         this._workspace.disconnect(this._windowRemovedId);
         global.display.disconnect(this._restackedId);
     }
-});
+}
 
-var WorkspaceIndicator = GObject.registerClass(
-class WorkspaceIndicator extends PanelMenu.Button {
-    _init() {
-        super._init(0.0, _('Workspace Indicator'), true);
+var WorkspaceIndicator = class WorkspaceIndicator extends PanelMenu.Button {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(0.0, _('Workspace Indicator'), true);
         this.setMenu(new PopupMenu.PopupMenu(this, 0.0, St.Side.BOTTOM));
         this.add_style_class_name('window-list-workspace-indicator');
+        this.remove_style_class_name('panel-button');
         this.menu.actor.remove_style_class_name('panel-menu');
 
         let container = new St.Widget({
@@ -244,13 +301,13 @@ class WorkspaceIndicator extends PanelMenu.Button {
             workspaceManager.connect_after('workspace-switched',
                 this._onWorkspaceSwitched.bind(this)),
             workspaceManager.connect('notify::layout-rows',
-                this._onWorkspaceOrientationChanged.bind(this)),
+                this._updateThumbnailVisibility.bind(this)),
         ];
 
         this.connect('scroll-event', this._onScrollEvent.bind(this));
         this._updateMenu();
         this._updateThumbnails();
-        this._onWorkspaceOrientationChanged();
+        this._updateThumbnailVisibility();
 
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.desktop.wm.preferences' });
         this._settingsChangedId = this._settings.connect(
@@ -269,12 +326,15 @@ class WorkspaceIndicator extends PanelMenu.Button {
         super._onDestroy();
     }
 
-    _onWorkspaceOrientationChanged() {
-        let vertical = global.workspace_manager.layout_rows === -1;
-        this.reactive = vertical;
+    _updateThumbnailVisibility() {
+        const { workspaceManager } = global;
+        const vertical = workspaceManager.layout_rows === -1;
+        const useMenu =
+            vertical || workspaceManager.n_workspaces > MAX_THUMBNAILS;
+        this.reactive = useMenu;
 
-        this._statusBin.visible = vertical;
-        this._thumbnailsBox.visible = !vertical;
+        this._statusBin.visible = useMenu;
+        this._thumbnailsBox.visible = !useMenu;
     }
 
     _onWorkspaceSwitched() {
@@ -290,6 +350,7 @@ class WorkspaceIndicator extends PanelMenu.Button {
     _nWorkspacesChanged() {
         this._updateMenu();
         this._updateThumbnails();
+        this._updateThumbnailVisibility();
     }
 
     _updateMenuOrnament() {
@@ -386,5 +447,4 @@ class WorkspaceIndicator extends PanelMenu.Button {
         let newIndex = this._currentWorkspace + diff;
         this._activate(newIndex);
     }
-});
-
+};
